#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "yopyfunc.h"
#include "usb.h"

#include <sys/stat.h>
#include <errno.h>

#define OK 1
#define NOK 0

/* time out in milliseconds  10 minutes*/
#define TIMEOUT (10*60*1000)

void update_progress(GtkProgressBar *progress, int file_size, int current)
{
  gchar percent[5];
  g_sprintf(percent, "%d %%", current*100/file_size);

  gtk_progress_bar_set_fraction(progress, 1.0/file_size*current);
  
  gtk_progress_bar_set_text(progress, percent);
}

int write_header(usb_dev_handle *yopy, const char *header, int file_size)
{
    int i, size;
    char *tmp = calloc(1, 20 * sizeof(char) );
    if ( tmp == NULL )
    {
        return NOK;
    }

    memcpy(tmp, header, strlen(header) );

    for (i = 0; i < 4; i++ )/*filesize*/
    {
        tmp[19-i] = file_size & 0xFF;
        file_size >>= 8;
    }

    size = usb_bulk_write(yopy, 1, tmp, 20, TIMEOUT);

    free(tmp);

    if ( size != 20 )
    {
        return NOK;
    }
    return OK;
}

int write_image(usb_dev_handle *yopy, FILE *file, int file_size, GtkProgressBar *progress)
{
    int size, total = 0, w;
    char *buf = malloc(1000 * sizeof(char));

    printf("%f\n", 1.0/file_size);
/*    gtk_progress_bar_set_pulse_step(progress, 1.0/(file_size/10000.0));*/

    if ( buf == NULL )
    {
        return NOK;
    }

    while ( g_main_context_iteration(NULL, FALSE) )
    {
        size = fread(buf, 1, 1000, file);
        if ( size <=0 )
        {
            break;
        }

        w  = usb_bulk_write(yopy, 1, buf, size, TIMEOUT);
        if ( w != size )
        {
            free(buf);
            return NOK;
        }
        total = total + size;
        update_progress(progress, file_size, total);
    }
    fprintf(stdout, "\nDone.\n");
    free(buf);
    return OK;
}

/**
*
* Protocol description:
*     (Protocol Reverse engineering was done by looking the output of
*     usb protocol sniffer when using Windows tool to upgrade Yopy's kernel
*     and filesystem )
*
*     Kernel:
*         1. Upload 20 bytes to Yopy where first 14 bytes contains ascii
*            presentation of header key YPKERNELUPDATE and last 4 bytes contains
*            the image size
*         2. Upload kernel image that is splitted in 1000 bytes blocks
*            to the Yopy
*
*     Filesystem:
*         1. Upload 20 bytes to Yopy where first 15 bytes contains ascii
*            presentation of header key YPFILESYSUPDATE and last 4 bytes contains
*            the image size
*         2. Upload filesystem image that is splitted in 1000 bytes
*            blocks to the Yopy
*/
int yopy_upgrade(const char *filename, const char *header, usb_dev_handle *yopy, int wait, GtkProgressBar *progress)
{
    FILE *file;
    struct stat stat_buf;
    int file_size;
    file = fopen(filename, "r");
    if ( file == NULL )
    {
        fprintf(stderr,"%s: %s\n", filename, strerror(errno));
        return NOK;
    }

    fstat( fileno(file), &stat_buf);
    file_size =  stat_buf.st_size;

    fprintf(stdout,"Uploading header key to Yopy.\n");
    if ( wait )
    {
        fprintf(stdout, "Waiting for previous flash update to be completed in Yopy.\n");
    }
    /* first write header key to Yopy */
    if ( write_header(yopy, header, file_size) != OK )
    {
        fprintf(stderr,"Something went wrong when uploading header key to Yopy.\n");
        fprintf(stderr,"%s\n", usb_strerror());
        fprintf(stderr,"Maybe you don't have permissions to write in the usb device\n");
        fclose(file);
        return NOK;
    }

    fprintf(stdout,"Uploading %s image file to Yopy.\n", filename);
    if ( write_image(yopy, file, file_size, progress) != OK )
    {
        fprintf(stderr, "Something went wrong when uploading the image.");
        fprintf(stderr, "%s\n", usb_strerror());
        fclose(file);
        return NOK;
    }

    fclose(file);
    return OK;
}

/**
*  Try to find specific vendor id and product number
*  and open that device
*/
struct usb_dev_handle *open_yopy(void)
{
    struct usb_bus *bus;
    struct usb_device *dev;

    usb_find_busses();
    usb_find_devices();

    for (bus = usb_busses; bus; bus = bus->next)
    {
        for (dev = bus->devices; dev; dev = dev->next)
        {
            if ( dev->descriptor.idVendor == 0xE7E && dev->descriptor.idProduct == 1 )
            {
                return usb_open(dev);
            }
        }
    }
    return NULL;
}




